package sun.nio.fs;

import sun.misc.Unsafe;

class NativeBuffers
{
    private static final Unsafe unsafe;
    private static final int TEMP_BUF_POOL_SIZE = 3;
    private static ThreadLocal<NativeBuffer[]> threadLocal;
    
    private NativeBuffers() {
    }
    
    static NativeBuffer allocNativeBuffer(int n) {
        if (n < 2048) {
            n = 2048;
        }
        return new NativeBuffer(n);
    }
    
    static NativeBuffer getNativeBufferFromCache(int n) {
        NativeBuffer[] array = (NativeBuffer[])NativeBuffers.threadLocal.get();
        if (array != null) {
            int i = 0;
            while (i < 3) {
                NativeBuffer nativeBuffer = array[i];
                if (nativeBuffer != null && nativeBuffer.size() >= n) {
                    array[i] = null;
                    return nativeBuffer;
                }
                ++i;
            }
        }
        return null;
    }
    
    static NativeBuffer getNativeBuffer(int n) {
        NativeBuffer nativeBufferFromCache = getNativeBufferFromCache(n);
        if (nativeBufferFromCache != null) {
            nativeBufferFromCache.setOwner(null);
            return nativeBufferFromCache;
        }
        return allocNativeBuffer(n);
    }
    
    static void releaseNativeBuffer(NativeBuffer nativeBuffer) {
        NativeBuffer[] array = (NativeBuffer[])NativeBuffers.threadLocal.get();
        if (array == null) {
            NativeBuffers.threadLocal.set(new NativeBuffer[] { nativeBuffer, null, null });
            return;
        }
        int i = 0;
        while (i < 3) {
            if (array[i] == null) {
                array[i] = nativeBuffer;
                return;
            }
            ++i;
        }
        int j = 0;
        while (j < 3) {
            NativeBuffer nativeBuffer2 = array[j];
            if (nativeBuffer2.size() < nativeBuffer.size()) {
                nativeBuffer2.cleaner().clean();
                array[j] = nativeBuffer;
                return;
            }
            ++j;
        }
        nativeBuffer.cleaner().clean();
        return;
    }
    
    static void copyCStringToNativeBuffer(byte[] srcBase, NativeBuffer nativeBuffer) {
        long srcOffset = (long)Unsafe.ARRAY_BYTE_BASE_OFFSET;
        long bytes = (long)srcBase.length;
        assert (long)nativeBuffer.size() >= bytes + 1L;
        NativeBuffers.unsafe.copyMemory(srcBase, srcOffset, null, nativeBuffer.address(), bytes);
        NativeBuffers.unsafe.putByte(nativeBuffer.address() + bytes, (byte)0);
        return;
    }
    
    static NativeBuffer asNativeBuffer(byte[] array) {
        NativeBuffer nativeBuffer = getNativeBuffer(array.length + 1);
        copyCStringToNativeBuffer(array, nativeBuffer);
        return nativeBuffer;
    }
    
    static {
        unsafe = Unsafe.getUnsafe();
        NativeBuffers.threadLocal = new ThreadLocal();
    }
}
